

######################
# internal functions #
######################


QPforSig <- function(counts, signatures) {

    require(Matrix)
    require(quadprog)

    if (!is.list(signatures)) {
        error("Error: signatures must be a list for function QPforSig")
    }

    # if necessary, convert signatures from matrices to vectors
    if (class(signatures[[1]]) == "matrix" || class(signatures[[1]]) == "data.frame") {
        signatures <- lapply(signatures, function(x){as.vector(t(as.matrix(x)))})
    }

    # if necessary, convert counts from matrix to vector
    if(class(counts) == "matrix" || class(counts) == "data.frame") {
        counts <- as.vector(t(as.matrix(counts)))
    }

    sigMa <- do.call(rbind,signatures) # the signatures should be stored in a matrix, one row is one signature
    #Rinverse <- backsolve(chol(sigMa %*% t(sigMa)),diag(dim(sigMa)[1]))
    sigMaSquared.nearPD = nearPD((sigMa %*% t(sigMa)), corr=FALSE, keepDiag=TRUE, ensureSymmetry=TRUE)$mat
    Rinverse <- backsolve(chol(sigMaSquared.nearPD),diag(dim(sigMa)[1]))
    
    Amat <- cbind(rep(1,dim(Rinverse)[1]), diag(dim(Rinverse)[1]))
    bvec <- c(1,rep(0,dim(Rinverse)[1]))

    dvec <- (counts) %*% t(sigMa)
    myQP <- quadprog::solve.QP(Dmat= Rinverse, dvec = dvec, Amat = Amat,bvec=bvec, meq = 1, factorized = TRUE)
    myQP$solution[myQP$solution<0]=0

    return(myQP$solution)
}


